// one big bad include file for the whole engine... nasty!

#include "tools.h"			

enum                            // block types, order matters!
{
    SOLID = 0,                  // entirely solid cube [only specifies wtex]
    CORNER,                     // half full corner of a wall
    FHF,                        // floor heightfield using neighbour vdelta values
    CHF,                        // idem ceiling
    SPACE,                      // entirely empty cube
    SEMISOLID,                  // generated by mipmapping
    MAXTYPE
};
 
struct sqr
{
    uchar type;                 // one of the above
    char floor, ceil;           // height, in cubes
    uchar wtex, ftex, ctex;     // wall/floor/ceil texture ids
    uchar r, g, b;              // light value at upper left vertex
    uchar vdelta;               // vertex delta, used for heightfield cubes
    char defer;                 // used in mipmapping, when true this cube is not a perfect mip
    char occluded;              // true when occluded
    uchar utex;                 // upper wall tex id
    uchar tag;                  // used by triggers
};

enum                            // hardcoded texture numbers
{
    DEFAULT_SKY = 0,
    DEFAULT_LIQUID,
    DEFAULT_WALL,
    DEFAULT_FLOOR,
    DEFAULT_CEIL
};

enum                            // static entity types
{
    NOTUSED = 0,                // entity slot not in use in map
    LIGHT,                      // lightsource, attr1 = radius, attr2 = intensity
    PLAYERSTART,                // attr1 = angle
    I_SHELLS, I_BULLETS, I_ROCKETS, I_ROUNDS,	/* JJT - 3,4,5,6 */
    I_HEALTH, I_BOOST,				/* JJT - 7,8 */
    I_GREENARMOUR, I_YELLOWARMOUR,		/* JJT - 9,10 */
    I_QUAD,					/* JJT - 11 */
    TELEPORT,                   // attr1 = idx
    TELEDEST,                   // attr1 = angle, attr2 = idx
    MAPMODEL,                   // attr1 = angle, attr2 = idx
    MONSTER,                    // attr1 = angle, attr2 = monstertype
    CARROT,                     // attr1 = tag, attr2 = type
    JUMPPAD,                    // attr1 = zpush, attr2 = ypush, attr3 = xpush
	COMPUTER,		/* JJT - 18: Computer Terminal */
	HOSPITAL,		/* JJT - 19: Medical Hospital/Terminal */
	GYM,			/* JJT - 20: Place to work out */
	HOME,			/* JJT - 21: Bed and safety */
	SYSOP,			/* JJT - 22: Central Computer Center and SysOp's office */
	SPEEDER,		/* JJT - 23: Speeder station */
	PAWNSHOP,		/* JJT - 24: Pawn Shop */
	GUNSHOP,		/* JJT - 25: Weapon Store */
	UNIVERSITY,		/* JJT - 26: Learning Center */
	PRECINCT,		/* JJT - 27: Police Station */
	MINESHAFT,		/* JJT - 28: Mine Shaft */
	WARGAMES,		/* JJT - 29: War Games Room */
	ARMORSHOP,		/* JJT - 30: Armor Shop */
	CONTROLLER,		/* JJT - 31: Central Admin Office */
	SNACKBAR,		/* JJT - 32: Food and drinks */
	SOCIALBAR,		/* JJT - 33: Generic Bar */
	TRUCKSTOP,		/* JJT - 34: Basic Restaurant */
	RESTAURANT,		/* JJT - 35: Food and drinks */
	ARMYBARRACKS,		/* JJT - 36: Army Guys here */
	MARINEBARRACKS,		/* JJT - 37: Marine Guys here */
	TEMPAGENCY,		/* JJT - 38: All kinds here */
	CASINO,			/* JJT - 39: Not really used */
	BANK,			/* JJT - 40: Financial Institution. Loans? */
	LOCKER,			/* JJT - 41: Bank for stuff. Comination? */
    MAXENTTYPES			/* JJT - 42: MAXENTTYPES */
};

struct persistent_entity        // map entity
{
    short x, y, z;              // cube aligned position
    short attr1;
    uchar type;                 // type is one of the above
    uchar attr2, attr3, attr4;        
};

struct entity : public persistent_entity    
{
    bool spawned;               // the only dynamic state of a map entity
};

#define MAPVERSION 5            // bump if map format changes, see worldio.cpp

struct header                   // map file format header
{
    char head[4];               // "CUBE"
    int version;                // any >8bit quantity is a little indian
    int headersize;             // sizeof(header)
    int sfactor;                // in bits
    int numents;
    char maptitle[128];
    uchar texlists[3][256];
    int waterlevel;
    int reserved[15];
};

#define SWS(w,x,y,s) (&(w)[(y)*(s)+(x)])
#define SW(w,x,y) SWS(w,x,y,ssize)
#define S(x,y) SW(world,x,y)            // convenient lookup of a lowest mip cube
#define SMALLEST_FACTOR 6               // determines number of mips there can be
#define DEFAULT_FACTOR 8
#define LARGEST_FACTOR 11               // 10 is already insane
#define SOLID(x) ((x)->type==SOLID)
#define MINBORD 2                       // 2 cubes from the edge of the world are always solid
#define OUTBORD(x,y) ((x)<MINBORD || (y)<MINBORD || (x)>=ssize-MINBORD || (y)>=ssize-MINBORD)

struct vec { float x, y, z; };
struct block { int x, y, xs, ys; };
struct mapmodelinfo { int rad, h, zoff, snap; char *name; };

/* JJT - The following enum should probably be updated to include all
	weapons from Mines of Titan. That will affect code that uses
	NUMGUNS. NUMGUNS will probably be defined separately.
*/

enum {
	GUN_FIST = 0,		/* JJT - 0: Fist */
	GUN_SG,			/* JJT - 1: Shotgun */
	GUN_CG,			/* JJT - 2: Chaingun */
	GUN_RL,			/* JJT - 3: Rocket Launcher */
	GUN_RIFLE,		/* JJT - 4: Railgun/Sniper Rifle */
	GUN_FIREBALL,		/* JJT - 5: Hot and Toasty! */
	GUN_ICEBALL,		/* JJT - 6: Splash! */
	GUN_SLIMEBALL,		/* JJT - 7: Blowing your nose at me? Nasty! */
	GUN_BITE,		/* JJT - 8: Chomp! */
	NUMGUNS			/* JJT - 9: 9 original guns */
	};

/* JJT - This new dynent has been updated to include the following entries:
	Bank Account Data including balance, RPG Attributes, Skills.
*/

struct dynent                           // players & monsters
{
    vec o, vel;                         // origin, velocity
    float yaw, pitch, roll;             // used as vec in one place
    float maxspeed;                     // cubes per second, 24 for player
    bool outsidemap;                    // from his eyes
    bool inwater;
    bool onfloor, jumpnext;
    int move, strafe;
    bool k_left, k_right, k_up, k_down; // see input code  
    int timeinair;                      // used for fake gravity
    float radius, eyeheight, aboveeye;  // bounding box size
    int lastupdate, plag, ping;
    int lifesequence;                   // sequence id for each respawn, used in damage test
    int state;                          // one of CS_* below
    int frags;
    int health, armour, armourtype, quadmillis;
    int gunselect, gunwait;
    int lastaction, lastattackgun, lastmove;
    bool attacking;

	/* JJT - Give every living entity a 12x20 inventory array.
		12 Inventory Slots x 20 of that same item in each slot.

	struct inventoryent *inventory[12][20];

	struct inventoryent *righthand;		JJT - This item points to inventory
	
	struct inventoryent *lefthand;		JJT - A second item pointing to inventory

		JJT - Helmet, body, belt, backpack, left & right glove, left & right boot
	struct inventoryent *worn[8];		JJT - More inventory item pointers

	JJT - Ammo array may be replaced by inventory.
	*/
    int ammo[NUMGUNS];

    int monsterstate;                   // one of M_* below, M_NONE means human
    int mtype;                          // see monster.cpp
    dynent *enemy;                      // monster wants to kill this entity
    float targetyaw;                    // monster wants to look in this direction
    bool blocked, moving;               // used by physics to signal ai
    int trigger;                        // millis at which transition to another monsterstate takes place
    vec attacktarget;                   // delayed attacks
    int anger;                          // how many times already hit by fellow monster
    string name, team;

	// JJT - Individual entities will have individual current attributes.

	/* JJT - Banking and Money section. */
	int bankaccountnumber, bankaccountcode;	// JJT - Bank Account

	float credits;			// JJT - Normally between 0 and 999. Float for special values.
	int millicredits;		// JJT - cents
	int kilocredits, megacredits;	// JJT - Big bucks
	int gigacredits, teracredits;	// JJT - Massive Money!
	int petacredits;		// JJT - User should never get this.

	/* JJT - How well fed and watered the entity is. */
	int food, water;

	/* JJT - trainingexp: How ready the entity is to train Attributes or Skills. */
	int trainingexp;

	int agility, stamina, strength;	// JJT - Physical Attributes.
	int charisma, intel, wisdom;	// JJT - Mental attributes.

	/* JJT - Individual attributes will have individual experience levels.
	*/

	int agilityexp, staminaexp, strengthexp;	// JJT - Physical Experience.
	int charismaexp, intelexp, wisdomexp;		// JJT - Mental Experience.
	int faction;					// SP-Game team.
	int factionrep[10][10];		// JJT - Reputation among the different factions.

	/* JJT - Individual attributes will have individual maximum levels. */

	int agilitymax, staminamax, strengthmax;	// JJT - Physical Experience.
	int charismamax, intelmax, wisdommax;		// JJT - Mental Experience.

	/* JJT - There will probably be a need for a
		maximum reputation array.
	*/

	int factionrepmax[10][10];		// JJT - Reputation among the different factions.

	// JJT - Individual entities will have individual skills.

	// JJT - Mental Skills

	int administration, language, medical, programming, repair;

	// JJT - Physical Skills

	int armorskill, melee, mining, swimming;

	// JJT - Combat Skills

	int arcgun, autogun, blade, handgun, rifle;

	// JJT - Individual skills will have individual experience.

	// JJT - Mental Skill Experience.

	int adminexp, languageexp, medicalexp, programexp, repairexp;

	// JJT - Physical Skill Experience.

	int armorexp, meleeexp, miningexp, swimexp;

	// JJT - Combat Skills

	int arcgunexp, autogunexp, bladeexp, handgunexp, rifleexp;

	int showstatsheet, hidestats, StatSheetAdjust;
	unsigned int TimerTick, TimerCount;	/* JJT - For a special half-second timer
							in the player's game loop.
					*/
};

/* #define SAVEGAMEVERSION 4               // bump if dynent/netprotocol changes or any other savegame/demo data
*/
#define SAVEGAMEVERSION 6	// JJT - New Save Game Version

/* enum { A_BLUE, A_GREEN, A_YELLOW };     // armour types... take 20/40/60 % off

	JJT - Change possible Armor Types to reflect "Mines of Titan" Armors.
*/

enum {
	A_BLUE,			/* JJT - Original Armor: 0 by default */
	A_GREEN,		/* JJT - Original Armor: 1 */
	A_YELLOW,		/* JJT - Original Armor: 2 */
	A_FLAK,			/* JJT - 3 */
	A_REFLECT,		/* JJT - 4 */
	A_MESH,			/* JJT - 5 */
	A_HYDRO,		/* JJT - 6 */
	A_BATTLE,		/* JJT - 7 */
	A_GOLUM,		/* JJT - 8 */
	A_NUMARMOURS		/* JJT - 9 */
	};
	/* JJT - Keep original armors, just for compatibility.
	*/

enum {
	M_NONE = 0,
	M_SEARCH,		/* JJT - 1 */
	M_HOME,			/* JJT - 2 */
	M_ATTACKING,		/* JJT - 3 */
	M_PAIN,			/* JJT - 4 */
	M_SLEEP,		/* JJT - 5 */
	M_AIMING,		/* JJT - 6 */
	M_FLEE,			/* JJT - 7 */
	M_FLEEAIMING,		/* JJT - 8 */
	M_FLEEATTACK,		/* JJT - 9 */
	M_STRAFE,		/* JJT - 10 */
	M_STRAFEAIMING,		/* JJT - 11 */
	M_STRAFEATTACK		/* JJT - 12 */
	};  // monster states

#define MAXCLIENTS 64                  // in a multiplayer game, can be arbitrarily changed
#define MAXTRANS 5000                   // max amount of data to swallow in 1 go
#define CUBE_SERVER_PORT 28765
#define CUBE_SERVINFO_PORT 28766
#define PROTOCOL_VERSION 121            // bump when protocol changes

// network messages codes, c2s, c2c, s2c
enum
{
	SV_INITS2C = 0,		/* JJT - 0 */
	SV_INITC2S,		/* JJT - 1 */
	SV_POS,			/* JJT - 2 */
	SV_TEXT,		/* JJT - 3 */
	SV_SOUND,		/* JJT - 4 */
	SV_CDIS,		/* JJT - 5 */
	SV_DIED,		/* JJT - 6 */
	SV_DAMAGE,		/* JJT - 7 */
	SV_SHOT,		/* JJT - 8 */
	SV_FRAGS,		/* JJT - 9 */
	SV_TIMEUP,		/* JJT - 10 */
	SV_EDITENT,		/* JJT - 11 */
	SV_MAPRELOAD,		/* JJT - 12 */
	SV_ITEMACC,		/* JJT - 13 */
	SV_MAPCHANGE,		/* JJT - 14 */
	SV_ITEMSPAWN,		/* JJT - 15 */
	SV_ITEMPICKUP,		/* JJT - 16 */
	SV_DENIED,		/* JJT - 17 */
	SV_PING,		/* JJT - 18 */
	SV_PONG,		/* JJT - 19 */
	SV_CLIENTPING,		/* JJT - 20 */
	SV_GAMEMODE,		/* JJT - 21 */
	SV_EDITH,		/* JJT - 22 */
	SV_EDITT,		/* JJT - 23 */
	SV_EDITS,		/* JJT - 24 */
	SV_EDITD,		/* JJT - 25 */
	SV_EDITE,		/* JJT - 26 */
	SV_SENDMAP,		/* JJT - 27 */
	SV_RECVMAP,		/* JJT - 28 */
	SV_SERVMSG,		/* JJT - 29 */
	SV_ITEMLIST,		/* JJT - 30 */
	SV_EXT,			/* JJT - 31 */
	};     

enum {
	CS_ALIVE = 0,		/* JJT - 0 */
	CS_DEAD,		/* JJT - 1 */
	CS_LAGGED,		/* JJT - 2 */
	CS_EDITING		/* JJT - 3 */
	};

// hardcoded sounds, defined in sounds.cfg
enum
{
	S_JUMP = 0,		/* JJT - 0 */
	S_LAND,			/* JJT - 1 */
	S_RIFLE,		/* JJT - 2 */
	S_PUNCH1,		/* JJT - 3 */
	S_SG,			/* JJT - 4 */
	S_CG,			/* JJT - 5 */
	S_RLFIRE, S_RLHIT,	/* JJT - 6, 7 */
	S_WEAPLOAD,		/* JJT - 8 */
	S_ITEMAMMO, S_ITEMHEALTH, S_ITEMARMOUR, S_ITEMPUP, S_ITEMSPAWN,
	S_TELEPORT,		/* JJT - 14 */
	S_NOAMMO,		/* JJT - 15 */
	S_PUPOUT,		/* JJT - 16 */
	S_PAIN0, S_PAIN1, S_PAIN2, S_PAIN3, S_PAIN4, S_PAIN5, S_PAIN6,
	S_DIE1, S_DIE2,		/* JJT - 24, 25 */
	S_FLAUNCH, S_FEXPLODE,	/* JJT - 26, 27 */
	S_SPLASH1, S_SPLASH2,	/* JJT - 28, 29 */
	S_GRUNT1, S_GRUNT2,	/* JJT - 30, 31 */
	S_RUMBLE,		/* JJT - 32 */
	S_PAINO,		/* JJT - 33 */
	S_PAINR, S_DEATHR,	/* JJT - 34, 35 */
	S_PAINE, S_DEATHE,	/* JJT - 36, 37 */
	S_PAINS, S_DEATHS,	/* JJT - 38, 39 */
	S_PAINB, S_DEATHB,	/* JJT - 40, 41 */
	S_PAINP, S_PIGGR2,	/* JJT - 42, 43 */
	S_PAINH, S_DEATHH,	/* JJT - 44, 45 */
	S_PAIND, S_DEATHD,	/* JJT - 46, 47 */
	S_PIGR1, S_ICEBALL, S_SLIMEBALL,
	S_JUMPPAD,		/* JJT - 51 */
};

// vertex array format

struct vertex { float u, v, x, y, z; uchar r, g, b, a; }; 

typedef vector<dynent *> dvector;
typedef vector<char *> cvector;
typedef vector<int> ivector;

// globals ooh naughty

extern sqr *world, *wmip[];             // map data, the mips are sequential 2D arrays in memory
extern header hdr;                      // current map header
extern int sfactor, ssize;              // ssize = 2^sfactor
extern int cubicsize, mipsize;          // cubicsize = ssize^2
extern dynent *player1;                 // special client ent that receives input and acts as camera
extern dvector players;                 // all the other clients (in multiplayer)
extern bool editmode;
extern vector<entity> ents;             // map entities
extern vec worldpos;                    // current target of the crosshair in the world
extern int lastmillis;                  // last time
extern int curtime;                     // current frame time
extern int gamemode, nextmode;
extern int xtraverts;
extern bool demoplayback;

/* JJT - Custom externs.
*/
extern int GlobalArrayInputX, GlobalArrayInputY;
extern int GlobalArrayOutputX, GlobalArrayOutputY;
extern float GlobalRandomArray[10][10];

#define DMF 16.0f 
#define DAF 1.0f 
#define DVF 100.0f

#define VIRTW 2400                      // virtual screen size for text & HUD
#define VIRTH 1800
#define FONTH 64
#define PIXELTAB (VIRTW/12)

#define PI  (3.1415927f)
#define PI2 (2*PI)

	/* JJT - Custom Defines
	*/
#define MaxAttrCnt 32000
#define InvSlotCnt 15
#define FactionCnt 10
#define MoneyLimit 999
#define Kilo 1000
#define Mega 1000000
#define Giga 1000000000
#define Tera 1000000000000
#define Peta 1000000000000000
#define MaxMoney 999999999999999999.999

// simplistic vector ops
#define dotprod(u,v) ((u).x * (v).x + (u).y * (v).y + (u).z * (v).z)
#define vmul(u,f)    { (u).x *= (f); (u).y *= (f); (u).z *= (f); }
#define vdiv(u,f)    { (u).x /= (f); (u).y /= (f); (u).z /= (f); }
#define vadd(u,v)    { (u).x += (v).x; (u).y += (v).y; (u).z += (v).z; };
#define vsub(u,v)    { (u).x -= (v).x; (u).y -= (v).y; (u).z -= (v).z; };
#define vdist(d,v,e,s) vec v = s; vsub(v,e); float d = (float)sqrt(dotprod(v,v));
#define vreject(v,u,max) ((v).x>(u).x+(max) || (v).x<(u).x-(max) || (v).y>(u).y+(max) || (v).y<(u).y-(max))
#define vlinterp(v,f,u,g) { (v).x = (v).x*f+(u).x*g; (v).y = (v).y*f+(u).y*g; (v).z = (v).z*f+(u).z*g; }

#define sgetstr() { char *t = text; do { *t = getint(p); } while(*t++); }   // used by networking

#define m_noitems     (gamemode>=4)
#define m_noitemsrail (gamemode<=5)
#define m_arena       (gamemode>=8)
#define m_tarena      (gamemode>=10)
#define m_teammode    (gamemode&1 && gamemode>2)
#define m_sp          (gamemode<0)
#define m_dmsp        (gamemode==-1)
#define m_classicsp   (gamemode==-2)

#define m_spmot		(gamemode==-3)	/* JJT - New Game Mode if needed for Mines of Titan 2 */

#define isteam(a,b)   (m_teammode && strcmp(a, b)==0)

enum    // function signatures for script functions, see command.cpp
{
			/* JJT - Following: 0-3 */
    ARG_1INT, ARG_2INT, ARG_3INT, ARG_4INT,
    ARG_NONE,		/* JJT - 4 */
			/* JJT - Following: 5-8 */
    ARG_1STR, ARG_2STR, ARG_3STR, ARG_5STR,
    ARG_DOWN, ARG_DWN1,	/* JJT - 9,10 */
    ARG_1EXP, ARG_2EXP,	/* JJT - 11, 12 */
    ARG_1EST, ARG_2EST,	/* JJT - 13, 14 */
    ARG_VARI		/* JJT - 15 */
}; 

// nasty macros for registering script functions, abuses globals to avoid excessive infrastructure
#define COMMANDN(name, fun, nargs) static bool __dummy_##fun = addcommand(#name, (void (*)())fun, nargs)
#define COMMAND(name, nargs) COMMANDN(name, name, nargs)
#define VAR(name, min, cur, max) int name = variable(#name, min, cur, max, &name, NULL)
#define VARF(name, min, cur, max, body) void var_##name(); static int name = variable(#name, min, cur, max, &name, var_##name); void var_##name() { body; }

#define ATOI(s) strtol(s, NULL, 0)		// supports hexadecimal numbers

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#include "windows.h"
	#define _WINDOWS
	#ifndef __GNUC__
		#define ZLIB_DLL
	#endif
#else
	#include <dlfcn.h>
#endif

#include <time.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glext.h>

#include <SDL.h>
#include <SDL_image.h>

#include <enet/enet.h>

#include <zlib.h>

#include "protos.h"				// external function decls
#include "attributes.h"
